@@ -7,7 +7,6 @@
 
 #include <progress_ipc.h>
 #include <lua.h>
-#include <lua_util.h>
 #include <lauxlib.h>
 #include <string.h> 
 #include <ifaddrs.h>
@@ -24,6 +23,7 @@
 #include <stdlib.h>
 
 #include "auxiliar.h"
+#include "lua_compat.h"
 
 #define WAIT 1
 
