@@ -10,6 +10,8 @@
 # SPDX-License-Identifier:	GPL-2.0-only
 set -eu
 
+SCRIPT_DIR=$(cd "$(dirname "$0")" && pwd)
+
 _SUDO=sudo
 if [ "$(id -u)" = 0 ]; then
     _SUDO=
@@ -27,6 +29,7 @@ $_SUDO apt-get install -y \
     curl \
     libzmq3-dev \
     liblua5.2-dev \
+    libluajit-5.1-dev \
     libconfig-dev \
     libarchive-dev \
     libbtrfsutil-dev \
@@ -63,4 +66,17 @@ $_SUDO apt-get install -y \
     wget \
     python3
 
-$_SUDO ln -sf /usr/lib/x86_64-linux-gnu/pkgconfig/lua5.2.pc /usr/lib/x86_64-linux-gnu/pkgconfig/lua.pc
+# packages are too old in Ubuntu Jammy
+if ! grep -q UBUNTU_CODENAME=jammy /etc/os-release; then
+    apt-get install -y \
+        libebgenv-dev \
+        libmtd-dev \
+        libubi-dev \
+        libubootenv-dev \
+        libzck-dev
+else
+    "$SCRIPT_DIR/install-src-deps.sh"
+fi
+
+PC_FILE_DIR=$(pkg-config --variable=pcfiledir lua5.2)
+$_SUDO ln -sf "$PC_FILE_DIR/lua5.2.pc" "$PC_FILE_DIR/lua.pc"
