@@ -21,13 +21,3 @@
 /* Docker base URL */
 
 #define DOCKER_BASE_URL	DOCKER_SOCKET_URL DOCKER_API_VERSION "/"
-
-typedef enum {
-	DOCKER_IMAGE_LOAD,
-	DOCKER_IMAGE_DELETE,
-	DOCKER_IMAGE_PRUNE,
-	DOCKER_CONTAINER_CREATE,
-	DOCKER_CONTAINER_DELETE,
-	DOCKER_CONTAINER_START,
-	DOCKER_CONTAINER_STOP
-} docker_services_t;
