@@ -73,7 +73,7 @@ static int _fill_buffer(int fd, unsigned char *buf, unsigned int nbytes, unsigne
 			return -EFAULT;
 		}
 		if (len == 0) {
-			return 0;
+			return count;
 		}
 		if (checksum)
 			for (i = 0; i < len; i++)
@@ -434,29 +434,54 @@ static int zstd_step(void* state, void* buffer, size_t size)
 
 #endif
 
-static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nbytes, unsigned long *offs, unsigned long long seek,
-	int skip_file, int __attribute__ ((__unused__)) compressed,
-	uint32_t *checksum, unsigned char *hash, bool encrypted, const char *imgivt, writeimage callback)
+static int hash_compare(struct swupdate_digest *dgst, unsigned char *hash)
+{
+	/*
+	 * SHA256_HASH_LENGTH should be enough but openssl might write
+	 * up to EVP_MAX_MD_SIZE = 64 bytes (sha512 size)
+	 */
+	unsigned char md_value[64];
+	unsigned int md_len = 0;
+
+	if (swupdate_HASH_final(dgst, md_value, &md_len) < 0) {
+		return -EFAULT;
+	}
+	/*
+	 * Now check if the computed hash is equal
+	 * to the value retrieved from sw-descritpion
+	 */
+	if (md_len != SHA256_HASH_LENGTH || swupdate_HASH_compare(hash, md_value)) {
+#ifndef CONFIG_ENCRYPTED_IMAGES_HARDEN_LOGGING
+		char hashstring[2 * SHA256_HASH_LENGTH + 1];
+		char newhashstring[2 * SHA256_HASH_LENGTH + 1];
+
+		hash_to_ascii(hash, hashstring);
+		hash_to_ascii(md_value, newhashstring);
+
+		ERROR("HASH mismatch : %s <--> %s",
+		      hashstring, newhashstring);
+#endif
+		return -EFAULT;
+	}
+	return 0;
+}
+
+int copyfile(struct swupdate_copy *args)
 {
 	unsigned int percent, prevpercent = 0;
 	int ret = 0;
 	int len;
-	unsigned char md_value[64]; /*
-				     *  Maximum hash is 64 bytes for SHA512
-				     *  and we use sha256 in swupdate
-				     */
-	unsigned int md_len = 0;
 	unsigned char *aes_key = NULL;
 	unsigned char *ivt = NULL;
 	unsigned char ivtbuf[AES_BLK_SIZE];
 
 	struct InputState input_state = {
-		.fdin = fdin,
+		.fdin = args->fdin,
 		.source = INPUT_FROM_FD,
 		.inbuf = NULL,
 		.pos = 0,
-		.nbytes = nbytes,
-		.offs = offs,
+		.nbytes = args->nbytes,
+		.offs = args->offs,
 		.dgst = NULL,
 		.checksum = 0
 	};
@@ -495,32 +520,33 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 	/*
 	 * If inbuf is set, read from buffer instead of from file
 	 */
-	if (inbuf) {
-		input_state.inbuf = inbuf;
+	if (args->inbuf) {
+		input_state.inbuf = args->inbuf;
 		input_state.source = INPUT_FROM_MEMORY;
 	}
 
 	PipelineStep step = NULL;
 	void *state = NULL;
 	uint8_t buffer[BUFF_SIZE];
+	writeimage callback = args->callback;
 
 	if (!callback) {
 		callback = copy_write;
 	}
 
-	if (checksum)
-		*checksum = 0;
+	if (args->checksum)
+		*args->checksum = 0;
 
-	if (IsValidHash(hash)) {
+	if (IsValidHash(args->hash)) {
 		input_state.dgst = swupdate_HASH_init(SHA_DEFAULT);
 		if (!input_state.dgst)
 			return -EFAULT;
 	}
 
-	if (encrypted) {
+	if (args->encrypted) {
 		aes_key = get_aes_key();
-		if (imgivt && strlen(imgivt)) {
-			if (!is_hex_str(imgivt) || ascii_to_bin(ivtbuf, sizeof(ivtbuf), imgivt)) {
+		if (args->imgivt && strlen(args->imgivt)) {
+			if (!is_hex_str(args->imgivt) || ascii_to_bin(ivtbuf, sizeof(ivtbuf), args->imgivt)) {
 				ERROR("Invalid image ivt");
 				return -EINVAL;
 			}
@@ -535,12 +561,12 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 		}
 	}
 
-	if (compressed) {
-		if (compressed == COMPRESSED_TRUE) {
+	if (args->compressed) {
+		if (args->compressed == COMPRESSED_TRUE) {
 			WARN("compressed argument: boolean form is deprecated, use compressed = \"zlib\";");
 		}
 #ifdef CONFIG_GUNZIP
-		if (compressed == COMPRESSED_ZLIB || compressed == COMPRESSED_TRUE) {
+		if (args->compressed == COMPRESSED_ZLIB || args->compressed == COMPRESSED_TRUE) {
 			/*
 			 * 16 + MAX_WBITS means that Zlib should expect and decode a
 			 * gzip header.
@@ -556,7 +582,7 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 		} else
 #endif
 #ifdef CONFIG_ZSTD
-		if (compressed == COMPRESSED_ZSTD) {
+		if (args->compressed == COMPRESSED_ZSTD) {
 			if ((zstd_state.dctx = ZSTD_createDStream()) == NULL) {
 				ERROR("ZSTD_createDStream failed");
 				ret = -EFAULT;
@@ -568,53 +594,44 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 		} else
 #endif
 		{
-			TRACE("Requested decompression method (%d) is not configured!", compressed);
+			TRACE("Requested decompression method (%d) is not configured!", args->compressed);
 			ret = -EINVAL;
 			goto copyfile_exit;
 		}
 	}
 
-	if (seek) {
-		int fdout = (out != NULL) ? *(int *)out : -1;
+	if (args->seek) {
+		int fdout = (args->out != NULL) ? *(int *)args->out : -1;
 		if (fdout < 0) {
 			ERROR("out argument: invalid fd or pointer");
 			ret = -EFAULT;
 			goto copyfile_exit;
 		}
 
-		TRACE("offset has been defined: %llu bytes", seek);
-		if (lseek(fdout, seek, SEEK_SET) < 0) {
+		TRACE("offset has been defined: %llu bytes", args->seek);
+		if (lseek(fdout, args->seek, SEEK_SET) < 0) {
 			ERROR("offset argument: seek failed");
 			ret = -EFAULT;
 			goto copyfile_exit;
 		}
 	}
 
+	step = &input_step;
+	state = &input_state;
+
+	if (args->encrypted) {
+		decrypt_state.upstream_step = step;
+		decrypt_state.upstream_state = state;
+		step = &decrypt_step;
+		state = &decrypt_state;
+	}
+
 #if defined(CONFIG_GUNZIP) || defined(CONFIG_ZSTD)
-	if (compressed) {
-		if (encrypted) {
-			decrypt_state.upstream_step = &input_step;
-			decrypt_state.upstream_state = &input_state;
-			decompress_state.upstream_step = &decrypt_step;
-			decompress_state.upstream_state = &decrypt_state;
-		} else {
-			decompress_state.upstream_step = &input_step;
-			decompress_state.upstream_state = &input_state;
-		}
+	if (args->compressed) {
+		decompress_state.upstream_step = step;
+		decompress_state.upstream_state = state;
 		step = decompress_step;
 		state = &decompress_state;
-	} else {
-#endif
-		if (encrypted) {
-			decrypt_state.upstream_step = &input_step;
-			decrypt_state.upstream_state = &input_state;
-			step = &decrypt_step;
-			state = &decrypt_state;
-		} else {
-			step = &input_step;
-			state = &input_state;
-		}
-#if defined(CONFIG_GUNZIP) || defined(CONFIG_ZSTD)
 	}
 #endif
 
@@ -626,7 +643,7 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 		if (ret == 0) {
 			break;
 		}
-		if (skip_file) {
+		if (args->skip_file) {
 			continue;
 		}
 		len = ret;
@@ -636,53 +653,32 @@ static int __swupdate_copy(int fdin, unsigned char *inbuf, void *out, size_t nby
 		 * results corrupted. This lets the cleanup routine
 		 * to remove it
 		 */
-		if (callback(out, buffer, len) < 0) {
+		if (callback(args->out, buffer, len) < 0) {
 			ret = -ENOSPC;
 			goto copyfile_exit;
 		}
 
-		percent = (unsigned)(100ULL * (nbytes - input_state.nbytes) / nbytes);
+		percent = (unsigned)(100ULL * (args->nbytes - input_state.nbytes) / args->nbytes);
 		if (percent != prevpercent) {
 			prevpercent = percent;
 			swupdate_progress_update(percent);
 		}
 	}
 
-	if (IsValidHash(hash)) {
-		if (swupdate_HASH_final(input_state.dgst, md_value, &md_len) < 0) {
-			ret = -EFAULT;
-			goto copyfile_exit;
-		}
-
-
-		/*
-		 * Now check if the computed hash is equal
-		 * to the value retrieved from sw-descritpion
-		 */
-		if (md_len != SHA256_HASH_LENGTH || swupdate_HASH_compare(hash, md_value)) {
-			char hashstring[2 * SHA256_HASH_LENGTH + 1];
-			char newhashstring[2 * SHA256_HASH_LENGTH + 1];
-
-			hash_to_ascii(hash, hashstring);
-			hash_to_ascii(md_value, newhashstring);
-
-#ifndef CONFIG_ENCRYPTED_IMAGES_HARDEN_LOGGING
-			ERROR("HASH mismatch : %s <--> %s",
-				hashstring, newhashstring);
-#endif
-			ret = -EFAULT;
-			goto copyfile_exit;
-		}
+	if (IsValidHash(args->hash) && hash_compare(input_state.dgst, args->hash) < 0) {
+		ret = -EFAULT;
+		goto copyfile_exit;
 	}
 
-	if (!inbuf) {
-		ret = _fill_buffer(fdin, buffer, NPAD_BYTES(*offs), offs, checksum, NULL);
+	if (!args->inbuf) {
+		ret = _fill_buffer(args->fdin, buffer, NPAD_BYTES(*args->offs),
+				   args->offs, args->checksum, NULL);
 		if (ret < 0)
 			DEBUG("Padding bytes are not read, ignoring");
 	}
 
-	if (checksum != NULL) {
-		*checksum = input_state.checksum;
+	if (args->checksum != NULL) {
+		*args->checksum = input_state.checksum;
 	}
 
 	ret = 0;
@@ -708,57 +704,23 @@ copyfile_exit:
 	return ret;
 }
 
-int copyfile(int fdin, void *out, size_t nbytes, unsigned long *offs, unsigned long long seek,
-	int skip_file, int __attribute__ ((__unused__)) compressed,
-	uint32_t *checksum, unsigned char *hash, bool encrypted, const char *imgivt, writeimage callback)
-{
-	return __swupdate_copy(fdin,
-				NULL,
-				out,
-				nbytes,
-				offs,
-				seek,
-				skip_file,
-				compressed,
-				checksum,
-				hash,
-				encrypted,
-				imgivt,
-				callback);
-}
-
-int copybuffer(unsigned char *inbuf, void *out, size_t nbytes, int __attribute__ ((__unused__)) compressed,
-	unsigned char *hash, bool encrypted, const char *imgivt, writeimage callback)
-{
-	return __swupdate_copy(-1,
-				inbuf,
-				out,
-				nbytes,
-				NULL,
-				0,
-				0,
-				compressed,
-				NULL,
-				hash,
-				encrypted,
-				imgivt,
-				callback);
-}
-
 int copyimage(void *out, struct img_type *img, writeimage callback)
 {
-	return copyfile(img->fdin,
-			out,
-			img->size,
-			(unsigned long *)&img->offset,
-			img->seek,
-			0, /* no skip */
-			img->compressed,
-			&img->checksum,
-			img->sha256,
-			img->is_encrypted,
-			img->ivt_ascii,
-			callback);
+	struct swupdate_copy copy = {
+		.fdin = img->fdin,
+		.out = out,
+		.callback = callback,
+		.nbytes = img->size,
+		.offs = (unsigned long*)&img->offset,
+		.seek = img->seek,
+		.skip_file = 0,
+		.compressed = img->compressed,
+		.checksum = &img->checksum,
+		.hash = img->sha256,
+		.encrypted = img->is_encrypted,
+		.imgivt = img->ivt_ascii,
+	};
+	return copyfile(&copy);
 }
 
 int extract_cpio_header(int fd, struct filehdr *fhdr, unsigned long *offset)
@@ -810,58 +772,6 @@ int extract_img_from_cpio(int fd, unsigned long offset, struct filehdr *fdh)
 	return 0;
 }
 
-int cpio_scan(int fd, struct swupdate_cfg *cfg, off_t start)
-{
-	struct filehdr fdh;
-	unsigned long offset = start;
-	int file_listed;
-	uint32_t checksum;
-
-	while (1) {
-		file_listed = 0;
-		start = offset;
-		if (extract_cpio_header(fd, &fdh, &offset)) {
-			return -1;
-		}
-		if (strcmp("TRAILER!!!", fdh.filename) == 0) {
-			return 0;
-		}
-
-		struct img_type *img = NULL;
-		SEARCH_FILE(img, cfg->images, file_listed, start);
-		SEARCH_FILE(img, cfg->scripts, file_listed, start);
-		SEARCH_FILE(img, cfg->bootscripts, file_listed, start);
-
-		TRACE("Found file:\n\tfilename %s\n\tsize %lu\n\t%s",
-			fdh.filename,
-			fdh.size,
-			file_listed ? "REQUIRED" : "not required");
-
-		/*
-		 * use copyfile for checksum and hash verification, as we skip file
-		 * we do not have to provide fdout
-		 */
-		if (copyfile(fd, NULL, fdh.size, &offset, 0, 1, 0, &checksum, img ? img->sha256 : NULL,
-				false, NULL, NULL) != 0) {
-			ERROR("invalid archive");
-			return -1;
-		}
-
-		if (!swupdate_verify_chksum(checksum, &fdh)) {
-			return -1;
-		}
-
-		/* Next header must be 4-bytes aligned */
-		offset += NPAD_BYTES(offset);
-		if (lseek(fd, offset, SEEK_SET) < 0) {
-			ERROR("CPIO file corrupted : %s", strerror(errno));
-			return -1;
-		}
-	}
-
-	return 0;
-}
-
 bool swupdate_verify_chksum(const uint32_t chk1, struct filehdr *fhdr) {
 	bool ret = (chk1 == fhdr->chksum);
 	if (fhdr->format == CPIO_NEWASCII)
