@@ -73,7 +73,8 @@ pthread_cond_t stream_cond = PTHREAD_COND_INITIALIZER;
 
 static struct installer inst;
 
-static int extract_file_to_tmp(int fd, const char *fname, unsigned long *poffs, bool encrypted)
+static int extract_file_to_tmp(int fd, const char *fname, unsigned long *poffs,
+			       bool encrypted, int max_size)
 {
 	char output_file[MAX_IMAGE_FNAME];
 	struct filehdr fdh;
@@ -95,6 +96,11 @@ static int extract_file_to_tmp(int fd, const char *fname, unsigned long *poffs,
 		ERROR("Path too long: %s%s", TMPDIR, fdh.filename);
 		return -1;
 	}
+	if (max_size && fdh.size >= max_size) {
+		ERROR("%s size (%ld) exceeds configured max of %d, aborting",
+			fdh.filename, fdh.size, max_size);
+		return -1;
+	}
 	TRACE("Found file");
 	TRACE("\tfilename %s", fdh.filename);
 	TRACE("\tsize %u", (unsigned int)fdh.size);
@@ -103,8 +109,15 @@ static int extract_file_to_tmp(int fd, const char *fname, unsigned long *poffs,
 	if (fdout < 0)
 		return -1;
 
-	if (copyfile(fd, &fdout, fdh.size, poffs, 0, 0, 0, &checksum, NULL,
-		     encrypted, NULL, NULL) < 0) {
+	struct swupdate_copy copy = {
+		.fdin = fd,
+		.out = &fdout,
+		.nbytes = fdh.size,
+		.offs = poffs,
+		.checksum = &checksum,
+		.encrypted = encrypted,
+	};
+	if (copyfile(&copy) < 0) {
 		close(fdout);
 		return -1;
 	}
@@ -165,7 +178,8 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 		switch (status) {
 		/* Waiting for the first Header */
 		case STREAM_WAIT_DESCRIPTION:
-			if (extract_file_to_tmp(fd, SW_DESCRIPTION_FILENAME, &offset, encrypted_sw_desc) < 0 )
+			if (extract_file_to_tmp(fd, SW_DESCRIPTION_FILENAME, &offset,
+						encrypted_sw_desc, software->swdesc_max_size) < 0 )
 				return -1;
 
 			status = STREAM_WAIT_SIGNATURE;
@@ -174,7 +188,8 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 		case STREAM_WAIT_SIGNATURE:
 #ifdef CONFIG_SIGNED_IMAGES
 			snprintf(output_file, sizeof(output_file), "%s.sig", SW_DESCRIPTION_FILENAME);
-			if (extract_file_to_tmp(fd, output_file, &offset, false) < 0 )
+			if (extract_file_to_tmp(fd, output_file, &offset, false,
+						software->swdesc_max_size) < 0 )
 				return -1;
 #endif
 			snprintf(output_file, sizeof(output_file), "%s%s", TMPDIR, SW_DESCRIPTION_FILENAME);
@@ -210,8 +225,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 			}
 
 			struct imglist *list[] = {&software->images,
-						  &software->scripts,
-						  &software->bootscripts};
+						  &software->scripts};
 
 			for (unsigned int i = 0; i < ARRAY_SIZE(list); i++) {
 				skip = check_if_required(list[i], &fdh,
@@ -230,6 +244,13 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 			fdout = -1;
 			offset = 0;
 
+			struct swupdate_copy copy = {
+				.fdin = fd,
+				.out = &fdout,
+				.nbytes = fdh.size,
+				.offs = &offset,
+				.checksum = &checksum,
+			};
 			/*
 			 * If images are not streamed directly into the target
 			 * copy them into TMPDIR to check if it is all ok
@@ -243,7 +264,8 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 					close(fdout);
 					return -1;
 				}
-				if (copyfile(fd, &fdout, fdh.size, &offset, 0, 0, 0, &checksum, img->sha256, false, NULL, NULL) < 0) {
+				copy.hash = img->sha256;
+				if (copyfile(&copy) < 0) {
 					close(fdout);
 					return -1;
 				}
@@ -255,7 +277,8 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 				break;
 
 			case SKIP_FILE:
-				if (copyfile(fd, &fdout, fdh.size, &offset, 0, skip, 0, &checksum, NULL, false, NULL, NULL) < 0) {
+				copy.skip_file = 1;
+				if (copyfile(&copy) < 0) {
 					return -1;
 				}
 				if (!swupdate_verify_chksum(checksum, &fdh)) {
@@ -288,7 +311,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 							return -1;
 						}
 						/* Avoid trying to adjust again later */
-						part->install_directly = 1;
+						part->install_directly = true;
 					}
 				}
 				img->fdin = fd;
@@ -445,6 +468,17 @@ static int save_stream(int fdin, struct swupdate_cfg *software)
 		tmpsize += NPAD_BYTES(tmpsize);
 		tmpsize -= sizeof(struct new_ascii_header);
 
+		/*
+		 * This doubles with check in extract_file_to_tmp, but it does not make
+		 * sense to check after having copied everything once in tmpfd...
+		 */
+		if (software->swdesc_max_size && fdh.size >= software->swdesc_max_size) {
+			ERROR("sw-description size (%ld) exceeds configured max of %d, aborting",
+				fdh.size, software->swdesc_max_size);
+			ret = -EINVAL;
+			goto no_copy_output;
+		}
+
 		/*
 		 * copy the remaining bytes to have a complete cpio block
 		 */
@@ -457,14 +491,16 @@ static int save_stream(int fdin, struct swupdate_cfg *software)
 	lseek(tmpfd, 0, SEEK_SET);
 	offset = 0;
 
-	if (extract_file_to_tmp(tmpfd, SW_DESCRIPTION_FILENAME, &offset, encrypted_sw_desc) < 0) {
+	if (extract_file_to_tmp(tmpfd, SW_DESCRIPTION_FILENAME, &offset,
+				encrypted_sw_desc, software->swdesc_max_size) < 0) {
 		ERROR("%s cannot be extracted", SW_DESCRIPTION_FILENAME);
 		ret = -EINVAL;
 		goto no_copy_output;
 	}
 #ifdef CONFIG_SIGNED_IMAGES
 	snprintf(output_file, sizeof(output_file), "%s.sig", SW_DESCRIPTION_FILENAME);
-	if (extract_file_to_tmp(tmpfd, output_file, &offset, false) < 0 ) {
+	if (extract_file_to_tmp(tmpfd, output_file, &offset, false,
+				software->swdesc_max_size) < 0 ) {
 		ERROR("Signature cannot be extracted:%s", output_file);
 		ret = -EINVAL;
 		goto no_copy_output;
@@ -731,13 +767,6 @@ void *network_initializer(void *data)
 		swupdate_remove_directory(DATADST_DIR_SUFFIX);
 #endif
 
-		pthread_mutex_lock(&stream_mutex);
-		inst.status = IDLE;
-		inst.req.source = SOURCE_UNKNOWN;
-		pthread_mutex_unlock(&stream_mutex);
-		TRACE("Main thread sleep again !");
-		notify(IDLE, RECOVERY_NO_ERROR, INFOLEVEL, "Waiting for requests...");
-
 		/*
 		 * Last step, if no restart is required,
 		 * SWUpdate can send automatically the feedback.
@@ -768,7 +797,12 @@ void *network_initializer(void *data)
 			ipc_send_cmd(&msg);
 		}
 
-
+		pthread_mutex_lock(&stream_mutex);
+		inst.status = IDLE;
+		inst.req.source = SOURCE_UNKNOWN;
+		pthread_mutex_unlock(&stream_mutex);
+		TRACE("Main thread sleep again !");
+		notify(IDLE, RECOVERY_NO_ERROR, INFOLEVEL, "Waiting for requests...");
 	}
 
 	pthread_exit((void *)0);
