@@ -88,6 +88,7 @@ static struct option long_options[] = {
 	{"dry-run", no_argument, NULL, 'n'},
 	{"file", required_argument, NULL, 'f'},
 	{"get-root", no_argument, NULL, 'g'},
+	{"get-emmc-boot", required_argument, NULL, 'E'},
 	{"help", no_argument, NULL, 'h'},
 #ifdef CONFIG_HW_COMPATIBILITY
 	{"hwrevision", required_argument, NULL, 'H'},
@@ -144,6 +145,7 @@ static void usage(char *programname)
 		" -e, --select <software>,<mode> : Select software images set and source\n"
 		"                                  Ex.: stable,main\n"
 		" -g, --get-root                 : detect and print the root device and exit\n" 
+		" -E, --get-emmc-boot <device>   : read the boot partition (CSD register) for a /dev/mmcblkX device\n"
 		" -q, --accepted-select\n"
 		"            <software>,<mode>   : List for software images set and source\n"
 		"                                  that are accepted via IPC\n"
@@ -268,7 +270,6 @@ static void swupdate_init(struct swupdate_cfg *sw)
 	LIST_INIT(&sw->images);
 	LIST_INIT(&sw->hardware);
 	LIST_INIT(&sw->scripts);
-	LIST_INIT(&sw->bootscripts);
 	LIST_INIT(&sw->bootloader);
 	LIST_INIT(&sw->extprocs);
 	sw->cert_purpose = SSL_PURPOSE_DEFAULT;
@@ -360,14 +361,16 @@ static int read_globals_settings(void *elem, void *data)
 
 	char software_select[SWUPDATE_GENERAL_STRING_SIZE] = "";
 	GET_FIELD_STRING(LIBCFG_PARSER, elem, "select", software_select);
-	GET_FIELD_STRING(LIBCFG_PARSER, elem,
-				"gpg-home-dir", sw->gpg_home_directory);
-	GET_FIELD_STRING(LIBCFG_PARSER, elem,
-				"gpgme-protocol", sw->gpgme_protocol);
 	if (software_select[0] != '\0') {
 		/* by convention, errors in a configuration section are ignored */
 		(void)parse_image_selector(software_select, sw);
 	}
+	GET_FIELD_STRING(LIBCFG_PARSER, elem,
+				"gpg-home-dir", sw->gpg_home_directory);
+	GET_FIELD_STRING(LIBCFG_PARSER, elem,
+				"gpgme-protocol", sw->gpgme_protocol);
+	GET_FIELD_INT(LIBCFG_PARSER, elem, "sw-description-max-size",
+				&sw->swdesc_max_size);
 
 	return 0;
 }
@@ -479,7 +482,7 @@ int main(int argc, char **argv)
 #endif
 	memset(main_options, 0, sizeof(main_options));
 	memset(image_url, 0, sizeof(image_url));
-	strcpy(main_options, "vhni:e:gq:l:Lcf:p:P:o:s:N:R:MmB:");
+	strcpy(main_options, "vhni:e:E:gq:l:Lcf:p:P:o:s:N:R:MmB:");
 #ifdef CONFIG_MTD
 	strcat(main_options, "b:");
 #endif
@@ -521,12 +524,14 @@ int main(int argc, char **argv)
 	 * Parse once the command line just to find if a
 	 * configuration file is passed
 	 */
+	int fd;
 	while ((c = getopt_long(argc, argv, main_options,
 				long_options, NULL)) != EOF) {
 		switch (c) {
 		char *root;
+		int bootdev;
 		case 'f':
-			cfgfname = sdup(optarg);
+			cfgfname = strdup(optarg);
 			break;
 		case 'g':
 			root = get_root_device();
@@ -536,6 +541,18 @@ int main(int argc, char **argv)
 			}
 			exit(EXIT_SUCCESS);
 			break;
+		case 'E':
+			fd = open(optarg, O_RDONLY);
+			if (fd < 0)
+				exit(EXIT_FAILURE);
+			bootdev = emmc_get_active_bootpart(fd);
+			close(fd);
+			if (bootdev < 0) {
+				exit(EXIT_FAILURE);
+			}
+			printf("%d\n", bootdev);
+			exit(EXIT_SUCCESS);
+			break;
 		case 'l':
 			loglevel = strtoul(optarg, NULL, 10);
 			break;
@@ -554,7 +571,7 @@ int main(int argc, char **argv)
 	if (cfgfname == NULL) {
 		struct stat stbuf;
 		if (stat(CONFIG_DEFAULT_CONFIG_FILE, &stbuf) == 0) {
-			cfgfname = sdup(CONFIG_DEFAULT_CONFIG_FILE);
+			cfgfname = strdup(CONFIG_DEFAULT_CONFIG_FILE);
 		}
 	}
 
@@ -997,5 +1014,6 @@ int main(int argc, char **argv)
 	if (!opt_c && !opt_i)
 		pthread_join(network_daemon, NULL);
 
+	free(cfgfname);
 	return exit_code;
 }
