@@ -21,6 +21,7 @@
 #include "lauxlib.h"
 #include "lualib.h"
 #include "lua_util.h"
+#include "lua_compat.h"
 #include "util.h"
 #include "handler.h"
 #include "bootloader.h"
@@ -388,7 +389,7 @@ static int l_copy2file(lua_State *L)
 	}
 
 	struct img_type img = {};
-	uint32_t checksum = 0;
+	uint32_t image_checksum = img.checksum;
 
 	table2image(L, &img);
 	if (check_same_file(img.fdin, fdout)) {
@@ -397,18 +398,7 @@ static int l_copy2file(lua_State *L)
 		lua_pushstring(L, "Output file path is same as input file temporary path");
 		goto copyfile_exit;
 	}
-	int ret = copyfile(img.fdin,
-				 &fdout,
-				 img.size,
-				 (unsigned long *)&img.offset,
-				 img.seek,
-				 0, /* no skip */
-				 img.compressed,
-				 &checksum,
-				 img.sha256,
-				 img.is_encrypted,
-				 img.ivt_ascii,
-				 NULL);
+	int ret = copyimage(&fdout, &img, NULL /* default write callback */);
 	update_table(L, &img);
 	lua_pop(L, 1);
 
@@ -417,10 +407,10 @@ static int l_copy2file(lua_State *L)
 		lua_pushstring(L, strerror(errno));
 		goto copyfile_exit;
 	}
-	if ((img.checksum != 0) && (checksum != img.checksum)) {
+	if ((image_checksum != 0) && (image_checksum != img.checksum)) {
 		lua_pushinteger(L, -1);
 		lua_pushfstring(L, "Checksums WRONG! Computed 0x%d, should be 0x%d\n",
-						checksum, img.checksum);
+				img.checksum, image_checksum);
 		goto copyfile_exit;
 	}
 
@@ -435,6 +425,7 @@ copyfile_exit:
 static int istream_read_callback(void *out, const void *buf, size_t len)
 {
 	lua_State* L = (lua_State*)out;
+	lua_Number result;
 	if (len > LUAL_BUFFERSIZE) {
 		ERROR("I/O buffer size is larger than Lua's buffer size %d", LUAL_BUFFERSIZE);
 		return -1;
@@ -449,12 +440,23 @@ static int istream_read_callback(void *out, const void *buf, size_t len)
 	memcpy(buffer, buf, len);
 	luaL_addsize(&lbuffer, len);
 	luaL_pushresult(&lbuffer);
-	if (lua_pcall(L, 1, 0, 0) != LUA_OK) {
+	if (lua_pcall(L, 1, 1, 0) != LUA_OK) {
 		ERROR("Lua error in callback: %s", lua_tostring(L, -1));
 		lua_pop(L, 1);
 		return -1;
 	}
-	return 0;
+
+	/* retrieve result */
+	if (!lua_isnumber(L, -1)) {
+		ERROR("Lua Callback must return a number");
+		lua_pop(L, 1);
+		return -1;
+	}
+
+	result = lua_tonumber(L, -1);
+	lua_pop(L, 1);
+
+	return (int) result;
 }
 
 static int l_istream_read(lua_State* L)
@@ -463,24 +465,13 @@ static int l_istream_read(lua_State* L)
 	luaL_checktype(L, 2, LUA_TFUNCTION);
 
 	struct img_type img = {};
-	uint32_t checksum = 0;
+	uint32_t image_checksum = img.checksum;
 
 	lua_pushvalue(L, 1);
 	table2image(L, &img);
 	lua_pop(L, 1);
 
-	int ret = copyfile(img.fdin,
-				 L,
-				 img.size,
-				 (unsigned long *)&img.offset,
-				 img.seek,
-				 0, /* no skip */
-				 img.compressed,
-				 &checksum,
-				 img.sha256,
-				 img.is_encrypted,
-				 img.ivt_ascii,
-				 istream_read_callback);
+	int ret = copyimage(L, &img, istream_read_callback);
 
 	lua_pop(L, 1);
 	update_table(L, &img);
@@ -491,10 +482,10 @@ static int l_istream_read(lua_State* L)
 		lua_pushstring(L, strerror(errno));
 		return 2;
 	}
-	if ((img.checksum != 0) && (checksum != img.checksum)) {
+	if ((image_checksum != 0) && (image_checksum != img.checksum)) {
 		lua_pushinteger(L, -1);
 		lua_pushfstring(L, "Checksums WRONG! Computed 0x%d, should be 0x%d\n",
-						checksum, img.checksum);
+				img.checksum, image_checksum);
 		return 2;
 	}
 	lua_pushinteger(L, 0);
@@ -856,21 +847,27 @@ static int l_getroot(lua_State *L) {
 	root_dev_type type = ROOT_DEV_PATH;
 	char **root = NULL;
 	char *value = rootdev;
-	char* dev_path = getroot_dev_path((char*)"", rootdev);
+	char* dev_path = NULL;
 
-	root = string_split(rootdev, '=');
-	if (count_string_array((const char **)root) > 1) {
-		if (!strncmp(root[0], "UUID", strlen("UUID"))) {
-			type = ROOT_DEV_UUID;
-			dev_path = getroot_dev_path((char*)"/dev/disk/by-uuid/", root[1]);
-		} else if (!strncmp(root[0], "PARTUUID", strlen("PARTUUID"))) {
-			type = ROOT_DEV_PARTUUID;
-			dev_path = getroot_dev_path((char*)"/dev/disk/by-partuuid/", root[1]);
-		} else if (!strncmp(root[0], "PARTLABEL", strlen("PARTLABEL"))) {
-			type = ROOT_DEV_PARTLABEL;
-			dev_path = getroot_dev_path((char*)"/dev/disk/by-partlabel/", root[1]);
+	if (rootdev) {
+		dev_path = getroot_dev_path((char*)"", rootdev);
+		root = string_split(rootdev, '=');
+		if (count_string_array((const char **)root) > 1) {
+			if (!strncmp(root[0], "UUID", strlen("UUID"))) {
+				type = ROOT_DEV_UUID;
+				dev_path = getroot_dev_path((char*)"/dev/disk/by-uuid/", root[1]);
+			} else if (!strncmp(root[0], "PARTUUID", strlen("PARTUUID"))) {
+				type = ROOT_DEV_PARTUUID;
+				dev_path = getroot_dev_path((char*)"/dev/disk/by-partuuid/", root[1]);
+			} else if (!strncmp(root[0], "PARTLABEL", strlen("PARTLABEL"))) {
+				type = ROOT_DEV_PARTLABEL;
+				dev_path = getroot_dev_path((char*)"/dev/disk/by-partlabel/", root[1]);
+			}
+			value = root[1];
 		}
-		value = root[1];
+	} else {
+		type = ROOT_DEV_UNKNOWN;
+		value = (char *)"";
 	}
 	lua_newtable (L);
 	if (dev_path) {
@@ -1267,6 +1264,7 @@ static int luaopen_swupdate(lua_State *L)
 	lua_push_enum(L, "UUID", ROOT_DEV_UUID);
 	lua_push_enum(L, "PARTUUID", ROOT_DEV_PARTUUID);
 	lua_push_enum(L, "PARTLABEL", ROOT_DEV_PARTLABEL);
+	lua_push_enum(L, "PARTUNKNOWN", ROOT_DEV_UNKNOWN);
 	lua_settable(L, -3);
 
 	if (is_type(L, LUA_TYPE_HANDLER)) {
@@ -1511,7 +1509,7 @@ call_handler_exit:
 	return 2;
 }
 
-int lua_handlers_init(lua_State *L)
+static int lua_handlers_init(lua_State *L, struct dict *bootenv)
 {
 	static const char location[] =
 #if defined(CONFIG_EMBEDDED_LUA_HANDLER)
@@ -1536,6 +1534,12 @@ int lua_handlers_init(lua_State *L)
 		lua_settable(L, -3);
 		lua_pop(L, 2);
 		luaL_requiref(L, "swupdate", luaopen_swupdate, 1 );
+		if (bootenv) {
+			struct dict **udbootenv = lua_newuserdata(L, sizeof(struct dict*));
+			*udbootenv = bootenv;
+			luaL_setfuncs(L, l_swupdate_bootenv, 1);
+			lua_pop(L, 1); /* remove unused copy left on stack */
+		}
 		lua_pop(L, 1); /* remove unused copy left on stack */
 		/* try to load Lua handlers for the swupdate system */
 #if defined(CONFIG_EMBEDDED_LUA_HANDLER)
@@ -1569,12 +1573,9 @@ lua_State *lua_session_init(struct dict *bootenv)
 	lua_setglobal(L, "SWUPDATE_LUA_TYPE"); /* prime L as LUA_TYPE_PEMBSCR */
 	luaL_openlibs(L); /* opens the standard libraries */
 	luaL_requiref(L, "swupdate", luaopen_swupdate, 1 );
-	struct dict **udbootenv = lua_newuserdata(L, sizeof(struct dict*));
-	*udbootenv = bootenv;
-	luaL_setfuncs(L, l_swupdate_bootenv, 1);
-	lua_pop(L, 1); /* remove unused copy left on stack */
 
-	lua_handlers_init(L);
+	lua_handlers_init(L, bootenv);
+
 
 	return L;
 }
@@ -1582,7 +1583,7 @@ lua_State *lua_session_init(struct dict *bootenv)
 int lua_init(void)
 {
 	lua_State *L = luaL_newstate();
-	int res = lua_handlers_init(L);
+	int res = lua_handlers_init(L, NULL);
 	print_registered_handlers(false);
 	unregister_session_handlers();
 	lua_close(L);
