@@ -39,7 +39,7 @@
 #include "delta_handler.h"
 #include "multipart_parser.h"
 #include "zchunk_range.h"
-#include "chained_handler.h"
+#include "handler_helpers.h"
 #include "swupdate_image.h"
 
 #define DEFAULT_MAX_RANGES	150	/* Apache has default = 200 */
@@ -161,14 +161,14 @@ static int network_process_data(multipart_parser* p, const char *at, size_t leng
 					zck_get_chunk_number(priv->chunk),
 					priv->current.chunksize);
 			if (priv->current.chunksize != 0) {
-				ret = copybuffer(priv->current.buf,
-						 &priv->fdout,
-						 priv->current.chunksize,
-						 COMPRESSED_ZSTD,
-						 hash,
-						 0,
-						 NULL,
-						 NULL);
+				struct swupdate_copy copy = {
+					.inbuf = priv->current.buf,
+					.out = &priv->fdout,
+					.nbytes = priv->current.chunksize,
+					.compressed = COMPRESSED_ZSTD,
+					.hash = hash,
+				};
+				ret = copyfile(&copy);
 			} else
 				ret = 0; /* skipping, nothing to be copied */
 			/* Buffer can be discarged */
@@ -178,7 +178,7 @@ static int network_process_data(multipart_parser* p, const char *at, size_t leng
 			 * if an error occurred, stops
 			 */
 			if (ret) {
-				ERROR("copybuffer failed !");
+				ERROR("copyfile failed !");
 				priv->error_in_parser = true;
 				return -EFAULT;
 			}
@@ -845,8 +845,15 @@ static bool copy_existing_chunks(zckChunk **dstChunk, struct hnd_priv *priv)
 				zck_get_chunk_number(chunk),
 				start,
 				len);
-		ret = copyfile(priv->fdsrc, &priv->fdout, len, &offset, 0, 0, COMPRESSED_FALSE,
-				&checksum, hash, false, NULL, NULL);
+		struct swupdate_copy copy = {
+			.fdin = priv->fdsrc,
+			.out = &priv->fdout,
+			.nbytes = len,
+			.offs = &offset,
+			.checksum = &checksum,
+			.hash = hash,
+		};
+		ret = copyfile(&copy);
 
 		free(sha);
 		if (ret)
@@ -933,17 +940,16 @@ static int install_delta(struct img_type *img,
 		char *filesystem = diskformat_fs_detect(priv->srcdev);
 		if (filesystem) {
 			char* DATADST_DIR;
-			if (asprintf(&DATADST_DIR, "%s%s", get_tmpdir(), DATADST_DIR_SUFFIX) != -1)  {
-				if (!swupdate_mount(priv->srcdev, DATADST_DIR, filesystem)) {
+
+			DATADST_DIR = swupdate_temporary_mount(MNT_DATA, priv->srcdev, filesystem);
+			if (DATADST_DIR) {
 					struct statvfs vfs;
 					if (!statvfs(DATADST_DIR, &vfs)) {
 						TRACE("Detected filesystem %s, block size : %lu, %lu blocks =  %lu size",
 						       filesystem, vfs.f_frsize, vfs.f_blocks, vfs.f_frsize * vfs.f_blocks);
 						priv->srcsize = vfs.f_frsize * vfs.f_blocks;
 					}
-					swupdate_umount(DATADST_DIR);
-				}
-				free(DATADST_DIR);
+					swupdate_temporary_umount(DATADST_DIR);
 			}
 			free(filesystem);
 		}
@@ -999,18 +1005,7 @@ static int install_delta(struct img_type *img,
 		goto cleanup;
 	}
 
-	ret = copyfile(img->fdin,
-		&mem_fd,
-		img->size,
-		(unsigned long *)&img->offset,
-		img->seek,
-		0,
-		img->compressed,
-		&img->checksum,
-		img->sha256,
-		img->is_encrypted,
-		img->ivt_ascii,
-		NULL);
+	ret = copyimage(&mem_fd, img, NULL /* default write callback */);
 
 	if (ret != 0) {
 		ERROR("Error %d copying zchunk header, aborting.", ret);
