@@ -12,7 +12,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-#include <ctype.h>
 #include <sys/file.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
@@ -62,7 +61,8 @@ enum partfield {
 	PART_DOSTYPE,
 	PART_UUID,
 	PART_FLAG,
-	PART_FORCE
+	PART_FORCE,
+	PART_FSLABEL
 };
 
 const char *fields[] = {
@@ -75,6 +75,7 @@ const char *fields[] = {
 	[PART_UUID] = "partuuid",
 	[PART_FLAG] = "flag",
 	[PART_FORCE] = "force",
+	[PART_FSLABEL] = "fslabel"
 };
 
 struct partition_data {
@@ -84,6 +85,7 @@ struct partition_data {
 	char type[SWUPDATE_GENERAL_STRING_SIZE];
 	char name[SWUPDATE_GENERAL_STRING_SIZE];
 	char fstype[SWUPDATE_GENERAL_STRING_SIZE];
+	char fslabel[SWUPDATE_GENERAL_STRING_SIZE];
 	char dostype[SWUPDATE_GENERAL_STRING_SIZE];
 	char partuuid[UUID_STR_LEN];
 	int explicit_size;
@@ -1153,6 +1155,54 @@ handler_release:
 	return ret;
 }
 
+static int format_parts(struct hnd_priv priv, struct img_type *img, struct create_table *createtable)
+{
+	int ret = 0;
+
+	char *path = realpath(img->device, NULL);
+	if (!path)
+		path = strdup(img->device);
+
+	struct partition_data *part;
+	LIST_FOREACH(part, &priv.listparts, next)
+	{
+		ret = 0; /* reset result */
+		/*
+		 * priv.listparts counts partitions starting with 0,
+		 * but fdisk_partname expects the first partition having
+		 * the number 1.
+		 */
+		size_t partno = part->partno + 1;
+
+		if (!strlen(part->fstype))
+			continue; /* Don't touch partitions without fstype */
+
+		char *device = fdisk_partname(path, partno);
+
+		bool do_mkfs = true;
+		if (!createtable->parent && !part->force) {
+			/* only create fs if it does not exist */
+			do_mkfs = !diskformat_fs_exists(device, part->fstype);
+		}
+
+		if (do_mkfs) {
+			ret = diskformat_mkfs(device, part->fstype);
+		} else {
+			TRACE("Skipping mkfs on %s", device);
+		}
+
+		if (!ret && part->fslabel[0] != '\0') {
+			ret = diskformat_set_fslabel(device, part->fstype, part->fslabel);
+		}
+
+		free(device);
+		if (ret)
+			break;
+	}
+	free(path);
+	return ret;
+}
+
 static int diskpart(struct img_type *img,
 	void __attribute__ ((__unused__)) *data)
 {
@@ -1282,6 +1332,16 @@ static int diskpart(struct img_type *img,
 						part->force = strtobool(equal);
 						TRACE("Force flag explicitly mentioned, value %d", part->force);
 						break;
+					case PART_FSLABEL:
+#ifdef CONFIG_DISKPART_FORMAT
+						strncpy(part->fslabel, equal, sizeof(part->fslabel));
+						break;
+#else
+						ERROR("Partitions have fslabel entries but diskpart format is "
+						      "missing!");
+						ret = -EINVAL;
+						goto handler_exit;
+#endif
 					}
 				}
 			}
@@ -1409,59 +1469,16 @@ handler_release:
 #ifdef CONFIG_DISKPART_FORMAT
 	/* Create filesystems */
 	if (!ret) {
-		LIST_FOREACH(part, &priv.listparts, next) {
-			/*
-			 * priv.listparts counts partitions starting with 0,
-			 * but fdisk_partname expects the first partition having
-			 * the number 1.
-			 */
-			size_t partno = part->partno + 1;
-
-			if (!strlen(part->fstype))
-				continue;  /* Don't touch partitions without fstype */
-
-			char *path = NULL;
-			char *device = NULL;
-
-			path = realpath(img->device, NULL);
-			if (!path)
-				path = strdup(img->device);
-			device = fdisk_partname(path, partno);
-			free(path);
-
-			if (!createtable->parent && !part->force) {
-				/* Check if file system exists */
-				ret = diskformat_fs_exists(device, part->fstype);
-
-				if (ret < 0) {
-					free(device);
-					break;
-				}
-
-				if (ret) {
-					TRACE("Found %s file system on %s, skip mkfs",
-						  part->fstype, device);
-					ret = 0;
-					free(device);
-					continue;
-				}
-			}
-
-			ret = diskformat_mkfs(device, part->fstype);
-			free(device);
-			if (ret)
-				break;
-		}
+		ret = format_parts(priv, img, createtable);
 	}
 #endif
 
+	/* cleanup */
 	LIST_FOREACH_SAFE(part, &priv.listparts, next, tmp) {
 		LIST_REMOVE(part, next);
 		free(part);
 	}
-
-	if (createtable)
-		free(createtable);
+	free(createtable);
 
 	/*
 	 * Declare that handler has finished
