@@ -76,7 +76,6 @@ struct swupdate_cfg {
 	struct swver installed_sw_list;
 	struct imglist images;
 	struct imglist scripts;
-	struct imglist bootscripts;
 	struct dict bootloader;
 	struct dict vars;
 	struct dict accepted_set;
@@ -86,24 +85,8 @@ struct swupdate_cfg {
 	const char *embscript;
 	char gpg_home_directory[SWUPDATE_GENERAL_STRING_SIZE];
 	char gpgme_protocol[SWUPDATE_GENERAL_STRING_SIZE];
+	int swdesc_max_size;
 };
 
-#define SEARCH_FILE(img, list, found, offs) do { \
-	if (!found) { \
-		for (img = list.lh_first; img != NULL; \
-			img = img->next.le_next) { \
-			if (strcmp(img->fname, fdh.filename) == 0) { \
-				found = 1; \
-				img->offset = offs; \
-				img->provided = 1; \
-				img->size = fdh.size; \
-			} \
-		} \
-		if (!found) \
-			img = NULL; \
-	} \
-} while(0)
-
-int cpio_scan(int fd, struct swupdate_cfg *cfg, off_t start);
 struct swupdate_cfg *get_swupdate_cfg(void);
 void free_image(struct img_type *img);
