@@ -1,15 +1,12 @@
 /*
  * (C) Copyright 2023
- * Stefano Babic, sbabic@denx.de.
+ * Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
 
-#ifndef _SWUPDATE_VERSION_H
-#define _SWUPDATE_VERSION_H
+#pragma once
 
 #include "swupdate_settings.h"
 void get_sw_versions(swupdate_cfg_handle *handle, struct swupdate_cfg *sw);
 
-#endif
-
