@@ -144,7 +144,7 @@ static int parser_follow_link(parsertype p, void *cfg, void *elem,
 	for (int j = 0; j < count_string_array(nodes); j++) {
 		linknodes[j] = nodes[j];
 	}
-	if (!set_find_path(linknodes, ref, tmp)) {
+	if (!set_find_path(linknodes, ref, &tmp)) {
 		free(linknodes);
 		return -1;
 	}
@@ -420,6 +420,7 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 	GET_FIELD_STRING(p, elem, "filesystem", image->filesystem);
 	GET_FIELD_STRING(p, elem, "type", image->type);
 	GET_FIELD_STRING(p, elem, "data", image->type_data);
+	GET_FIELD_INT64(p, elem, "size", &image->size);
 	get_hash_value(p, elem, image->sha256);
 
 	/*
@@ -427,7 +428,7 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 	 * multiplier suffixes are allowed
 	 */
 	if (is_field_numeric(p, elem, "offset")) {
-		GET_FIELD_INT64(p, elem, "offset", &offset);
+		GET_FIELD_INT64(p, elem, "offset", (long long *)&offset);
 		image->seek = offset;
 	} else {
 		GET_FIELD_STRING(p, elem, "offset", seek_str);
@@ -515,13 +516,6 @@ static int _parse_partitions(parsertype p, void *cfg, void *setting, const char
 
 		partition->provided = 1;
 
-		if ((!strlen(partition->volname) && !strcmp(partition->type, "ubipartition")) ||
-				!strlen(partition->device)) {
-			ERROR("Partition incompleted in description file");
-			free_image(partition);
-			return -1;
-		}
-
 		GET_FIELD_INT64(p, elem, "size", &partition->partsize);
 
 		add_properties(p, elem, partition);
@@ -531,6 +525,18 @@ static int _parse_partitions(parsertype p, void *cfg, void *setting, const char
 			free_image(partition);
 			return -1;
 		}
+		if (skip || partition->skip != SKIP_NONE) {
+			free_image(partition);
+			continue;
+		}
+
+		if ((!strlen(partition->volname) && !strcmp(partition->type, "ubipartition")) ||
+				!strlen(partition->device)) {
+			ERROR("Partition incompleted in description file");
+			free_image(partition);
+			return -1;
+		}
+
 		TRACE("Partition: %s new size %lld bytes",
 			!strcmp(partition->type, "ubipartition") ? partition->volname : partition->device,
 			partition->partsize);
@@ -644,7 +650,6 @@ static int _parse_bootloader(parsertype p, void *cfg, void *setting, const char
 {
 	void *elem;
 	int count, i, skip, err;
-	struct img_type *script;
 	struct img_type dummy;
 
 	if (setting == NULL) {
@@ -671,58 +676,27 @@ static int _parse_bootloader(parsertype p, void *cfg, void *setting, const char
 		/*
 		 * Check for mandatory field
 		 */
-		if(exist_field_string(p, elem, "name")) {
-			/*
-			 * Call directly get_field_string with size 0
-			 * to let allocate the place for the strings
-			 */
-			GET_FIELD_STRING(p, elem, "name", dummy.id.name);
-			GET_FIELD_STRING(p, elem, "value", dummy.id.version);
-			skip = run_embscript(p, elem, &dummy, L, swcfg->embscript);
-			if (skip < 0) {
-				return -1;
-			}
-			if (!skip) {
-				dict_set_value(&swcfg->bootloader, dummy.id.name, dummy.id.version);
-				TRACE("Bootloader var: %s = %s",
-					dummy.id.name,
-					dict_get_value(&swcfg->bootloader, dummy.id.name));
-			}
-			continue;
-		}
-
-		/*
-		 * Check if it is a bootloader script
-		 */
-		if(!(exist_field_string(p, elem, "filename"))) {
-			TRACE("bootloader entry is neither a script nor name/value.");
-			continue;
-		}
-		script = (struct img_type *)calloc(1, sizeof(struct img_type));
-		if (!script) {
-			ERROR( "No memory: malloc failed");
-			return -ENOMEM;
-		}
-
-		if (parse_common_attributes(p, elem, script, swcfg) < 0) {
-			free_image(script);
+		if(!exist_field_string(p, elem, "name")) {
+			ERROR("No variable set in bootenv section");
 			return -1;
 		}
 
-		script->is_script = 1;
-
-		skip = run_embscript(p, elem, script, L, swcfg->embscript);
-		if (skip != 0 || script->skip != SKIP_NONE) {
-			free_image(script);
-			if (skip < 0)
-				return -1;
-			continue;
+		/*
+		 * Call directly get_field_string with size 0
+		 * to let allocate the place for the strings
+		 */
+		GET_FIELD_STRING(p, elem, "name", dummy.id.name);
+		GET_FIELD_STRING(p, elem, "value", dummy.id.version);
+		skip = run_embscript(p, elem, &dummy, L, swcfg->embscript);
+		if (skip < 0) {
+			return -1;
+		}
+		if (!skip) {
+			dict_set_value(&swcfg->bootloader, dummy.id.name, dummy.id.version);
+			TRACE("Bootloader var: %s = %s",
+				dummy.id.name,
+				dict_get_value(&swcfg->bootloader, dummy.id.name));
 		}
-
-		LIST_INSERT_HEAD(&swcfg->bootscripts, script, next);
-
-		TRACE("Found U-Boot Script: %s",
-			script->fname);
 	}
 
 	return 0;
@@ -1155,6 +1129,7 @@ int parse_json(struct swupdate_cfg *swcfg, const char *filename, char **error)
 	if (!cfg) {
 		if (asprintf(error, "JSON File corrupted") == ENOMEM_ASPRINTF) {
 			ERROR("OOM when caching error");
+			free(string);
 			return -ENOMEM;
 		}
 		free(string);
