@@ -873,6 +873,7 @@ const char *sym_expand_string_value(const char *in)
 {
 	const char *src;
 	char *res;
+	char *tmp_res;
 	size_t reslen;
 
 	reslen = strlen(in) + 1;
@@ -902,7 +903,10 @@ const char *sym_expand_string_value(const char *in)
 		newlen = strlen(res) + strlen(symval) + strlen(src) + 1;
 		if (newlen > reslen) {
 			reslen = newlen;
-			res = realloc(res, reslen);
+			tmp_res = realloc(res, reslen);
+			if (!tmp_res)
+				free(res);
+			res = tmp_res;
 		}
 
 		strcat(res, symval);
