@@ -867,22 +867,23 @@ static void get_action_id_from_env(int *action_id)
 	 * Get the acction_id that corresponds to the done update if it was
 	 * stored.
 	 */
-	*action_id = -1;
 	char *action_str = swupdate_vars_get("action_id", NULL);
-	if (action_str) {
-		int tmp = ustrtoull(action_str, NULL, 10);
-		if (errno)
-			WARN("action_id %s: ustrtoull failed",
-			     action_str);
-		/*
-		 * action_id = 0 is invalid, then check it
-		 */
-		if (tmp > 0) {
-			*action_id = tmp;
-			TRACE("Retrieve action_id from previous run: %d", *action_id);
-		}
-		free(action_str);
+	if (!action_str) {
+		WARN("Action id not in env: action from server sent, possible mismatch ");
+		return;
 	}
+	int tmp = ustrtoull(action_str, NULL, 10);
+	if (errno)
+		WARN("action_id %s: ustrtoull failed",
+		     action_str);
+	/*
+	 * action_id = 0 is invalid, then check it
+	 */
+	if (tmp > 0) {
+		*action_id = tmp;
+		TRACE("Retrieve action_id from previous run: %d", *action_id);
+	}
+	free(action_str);
 }
 
 server_op_res_t server_handle_initial_state(update_state_t stateovrrd)
@@ -1003,7 +1004,12 @@ static void *process_notification_thread(void *data)
 		bool data_avail = false;
 		int ret = ipc_get_status(&msg);
 
-		data_avail = ret > 0 && (strlen(msg.data.status.desc) != 0);
+		if (ret < 0) {
+			ERROR("Error getting status, stopping notification thread");
+			stop = true;
+		} else {
+			data_avail = (strlen(msg.data.status.desc) != 0);
+		}
 
 		/*
 		 * Mutex used to synchronize end of the thread
@@ -1017,12 +1023,6 @@ static void *process_notification_thread(void *data)
 
 		if (data_avail && msg.data.status.current == PROGRESS)
 			continue;
-		/*
-		 * If there is a message
-		 * ret > 0: data available
-		 * ret == 0: TIMEOUT, no more messages
-		 * ret < 0 : ERROR, exit
-		 */
 		if (data_avail && numdetails < MAX_DETAILS) {
 			for (int c = 0; c < strlen(msg.data.status.desc); c++) {
 				switch (msg.data.status.desc[c]) {
@@ -1730,10 +1730,10 @@ static int server_hawkbit_settings(void *elem, void  __attribute__ ((__unused__)
 	}
 
 	GET_FIELD_INT(LIBCFG_PARSER, elem, "polldelay",
-		&server_hawkbit.polling_interval);
+		(int *)&server_hawkbit.polling_interval);
 
 	GET_FIELD_INT(LIBCFG_PARSER, elem, "initial-report-resend-period",
-		&server_hawkbit.initial_report_resend_period);
+		(int *)&server_hawkbit.initial_report_resend_period);
 
 	channel_settings(elem, &channel_data_defaults);
 
@@ -1741,7 +1741,7 @@ static int server_hawkbit_settings(void *elem, void  __attribute__ ((__unused__)
 		&server_hawkbit.usetokentodwl);
 
 	GET_FIELD_INT(LIBCFG_PARSER, elem, "connection-timeout",
-		&channel_data_defaults.connection_timeout);
+		(int *)&channel_data_defaults.connection_timeout);
 
 	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "targettoken", tmp);
 	if (strlen(tmp))
